/* 
 * ANEPFlow.java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the ANTS distribution.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that (i) the above copyright notice(s)
 * appear in all copies and (ii) that both the copyright notice(s)
 * and this permission notice appear in supporting documentation,
 * and (iii) that the name(s) of the copyright holders not be used
 * in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  The
 * copyright holders make no representations about the suitability
 * of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 * 
 * The maintainers of this code ask, but do not require, that
 * improvements and bug fixes be emailed to ants-dist@cs.utah.edu
 *         
 *
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 */
package ants.core;

import edu.utah.janos.route.RouteEntry;

/**
 * Forwards unrecognized ANEP packets.
 * An ANEP Flow will provide default handling for ANEP packets
 * that are not of a recognized type.  The associated Channel
 * is connected to the ANEPDispatcher's registerDefault method.
 * <p>
 * XXX THIS DOES NOT WORK.  IT HAS NOT BEEN MIGRATED TO THE NODEOS APIS.
 *
 * @history Originally ANEPChannel.Java
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 * @author David Wetherall
 */
class ANEPFlow
{
    /**
     * Decode ANEP header and forward packet.  This is for non-ANTS
     * ANEP packets.
     * <p>
     * Always returns null because no capsule is created.
     * <p>
     * XXX should throw something more informative than 'Exception'
     * 
     * @param xdr The external representation of the ANEP packet.
     * @exception Exception thrown when anything goes wrong.
     * @return null
     */
    protected Capsule decode(Xdr xdr)
	throws Exception
    {
	// Make sure its an appropriate ANEP packet
	if (xdr.BYTE() != Capsule.ANEPVERSION)
	    throw new Exception("Wrong ANEP version");
	
	boolean discard = (xdr.BYTE() & 0x80) != 0;
	short typeID = xdr.SHORT();
	short headerLen = xdr.SHORT();
	xdr.skip(Xdr.SHORT);     // Skip packet length field
	
	if (typeID != Capsule.getAnepTypeID()) {
	    if (discard)
		throw new Exception("Unknown type discard");
	    else {
		int dest = 0;
		while (xdr.index() < headerLen * Xdr.INT) {
		    short flagsAndType = xdr.SHORT();
		    boolean optPrivate = (flagsAndType & 0x8000) != 0;
		    boolean optDiscard = (flagsAndType & 0x4000) != 0;	
		    short optType = (short)(xdr.SHORT() & 0x3FFF);
		    short optLen = xdr.SHORT();
		    if (!optPrivate) {
			if (optType == Capsule.DESTOPTTYPE) {
			    int scheme = xdr.INT();
			    if (scheme != Capsule.IPV4SCHEME)
				throw new Exception("Unknown addressing scheme");
			    else {
				dest = xdr.INT();
			    }
			} else if (optType == Capsule.SRCOPTTYPE) {
			    xdr.skip((optLen - 1) * Xdr.INT);
			} else if (optDiscard) 
			    throw new Exception("Unknown option discard");
		    } else {
			xdr.skip((optLen - 1) * Xdr.INT);
		    }
		}
		
		Node.abort("ANEPFlow not upgraded for getting routes and forwarding packets");
		
		/*
		RouteEntry r = Node.getRoutes().get(dest);
		if (r != null) {
		    if (xdr.offset != 0) {
			byte[] bytes = new byte[xdr.len];
			System.arraycopy(xdr.buf, xdr.offset, bytes, 0, xdr.len);
			Node.link.send(bytes, xdr.len, r.addr);
		    } else
			Node.link.send(xdr.buf, xdr.len, r.addr);
		    throw new Exception("Unknown type forwarded");
		} else
		    throw new Exception("No route to destination while forwarding");
		*/
	    }
	}
	// skip ANEP header
	xdr.skip(headerLen * Xdr.INT);
	return null;
    }
    
    /**
     * Construct an ANEPFlow.
     */
    ANEPFlow()
    {
	Node.abort("Dunno how to construct an ANEP Flow!");
    }
}
