/* 
 * BuiltinProtocol.java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the ANTS distribution.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that (i) the above copyright notice(s)
 * appear in all copies and (ii) that both the copyright notice(s)
 * and this permission notice appear in supporting documentation,
 * and (iii) that the name(s) of the copyright holders not be used
 * in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  The
 * copyright holders make no representations about the suitability
 * of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 * 
 * The maintainers of this code ask, but do not require, that
 * improvements and bug fixes be emailed to ants-dist@cs.utah.edu
 *         
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */
package ants.core;

import edu.utah.janos.nodeos.Flow;

import ants.util.MD5;

/**
 * Base class for all builtin protocols.  Builtin protocols are 
 * not demand loaded and are known by "well-defined" names.
 * <p>
 * Package access only.
 *
 * @author David Wetherall
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
class BuiltinProtocol
    extends Protocol
{
    /**
     * Override Protocol's builtin to flag this class as a builtin.
     */
    boolean builtin()
    {
	return true;
    }	
    
    /**
     * Builtin protocols must use this version of addCapsule.
     * This versions uses the given version string to generate
     * the MD5 hash for the code ID.   (Thus, the version of
     * a builtin protocol is not tied to the actual code.)
     * <p>
     * Each builtin protocol class has a version string associated
     * with it.  See, for example, DataCapsule.
     *
     * @see DataCapsule#VERSION
     */
    protected void addCapsule(String name, String version)
	throws Exception  // XXX
    {
	/*
	 * This is how the BuiltinProtocol.addCapsule() method
	 * differs from the Protocol version: compute codeID
	 * from the version string..
	 */
	MD5 id = new MD5();
	id.update(version);
	TypeID codeID = TypeID.wrap(id.hash()).intern();
	
	// Note that we don't bother loading the code for builtin
	// classes because it will (1) never be shipped anywhere
	// and (2) always loaded off the local disk before our
	// ClassLoader gets a chance to try it.
	
	CodeGroupData g = (CodeGroupData)groups.lastElement();
	g.classes.addElement(new CapsuleClassData(name, null, codeID));
	Flow.currentFlow().exportClass(name);
    }
    
    /**
     * Builtin protocols should never call this version of addCapsule.
     *
     * @see #addCapsule(java.lang.String,java.lang.String)
     */
    protected void addCapsule(String name)
	throws Exception // XXX
    {
	Node.abort("regular addCapsule() called on a builtin protocol!");
    }
}

