/*
 * CodeRequestDemultiplexKey.Java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the ANTS distribution.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that (i) the above copyright notice(s)
 * appear in all copies and (ii) that both the copyright notice(s)
 * and this permission notice appear in supporting documentation,
 * and (iii) that the name(s) of the copyright holders not be used
 * in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  The
 * copyright holders make no representations about the suitability
 * of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 * 
 * The maintainers of this code ask, but do not require, that
 * improvements and bug fixes be emailed to ants-dist@cs.utah.edu
 *         
 */
package ants.core;

import edu.utah.janos.nodeos.Flow;

/**
 * Match ANTS packets for a specific ANTS protocol plus an
 * additional MID after the standard ANTS header.  Used to
 * match MIDIDRequest, DLRequest, and DLResponse capsules.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
class CodeRequestDemultiplexKey
    extends ANTSDemultiplexKey
{
    /**
     * The superclass, ANTSDemultiplexKey, will match mid1 (the MID
     * from Capsule.Java), and then we have to skip mid2Skip bytes
     * to get to mid2 (the MID from
     * MIDIDRequest/DLRequest/DLResponse).
     * (We're skipping over the source/dest/prev addrs.)
     */
    private static final int mid2Skip =
	Capsule.HEADERLENGTH - Xdr.TYPEID;
    
    /**
     * XXX This is always equal to Xdr.TYPEID... right?
     */
    private final int mid2Length;
    
    /*package*/ CodeRequestDemultiplexKey(Flow flow,
					  byte mid1Bytes[],
					  byte mid2Bytes[])
    {
	/*
	 * Match a regular ANTS packet with the given MID.
	 */
	super(flow, mid1Bytes);
	this.mid2Length = mid2Bytes.length;
	
	/*
	 * Skip 
	 */
	this.add(mid2Skip, mid2Bytes.length, mid2Bytes, null);
    }
    
    public int whereDoesTheCapsuleDecodingActuallyStart(int payloadStart)
    {
	/* Back up over mid2 to the start of the payload */
	return payloadStart - mid2Skip - this.mid2Length;
    }
}
