/* 
 * DLProtocol.java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the ANTS distribution.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that (i) the above copyright notice(s)
 * appear in all copies and (ii) that both the copyright notice(s)
 * and this permission notice appear in supporting documentation,
 * and (iii) that the name(s) of the copyright holders not be used
 * in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  The
 * copyright holders make no representations about the suitability
 * of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 * 
 * The maintainers of this code ask, but do not require, that
 * improvements and bug fixes be emailed to ants-dist@cs.utah.edu
 *         
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */

package ants.core;

import ants.core.security.Principal;

/**
 * Demand Loading protocol definition
 *
 * @author David Wetherall
 */
final class DLProtocol
    extends BuiltinProtocol
{
    DLProtocol()
	throws Exception
    {
	startProtocolDefn();
	
	// Load all of the inert capsules
	startGroupDefn(false);
	addCapsule("ants.core" + ".DLRequestCapsule",
		   DLRequestCapsule.VERSION);
	addCapsule("ants.core" + ".DLResponseCapsule",
		   DLResponseCapsule.VERSION);
	
	addCapsule("ants.core" + ".MIDIDRequestCapsule",
		   MIDIDRequestCapsule.VERSION);
	endGroupDefn();
	
	
	startGroupDefn();
	addCapsule("ants.core" + ".MIDIDResponseCapsule",
		   MIDIDResponseCapsule.VERSION);
	endGroupDefn();
	
	endProtocolDefn();
    }
    
    /**
     * We override activate here since the unknown packet handler should
     * be part of this protocol, but since it isn't standard we need to
     * manually add it here...
     */
    void activate(Node node, boolean register, Principal principal)
	throws DuplicateCapsuleClassException,
	FullCodeGroupException,
	java.lang.ClassNotFoundException
    {
	if( !register )
	    new UnknownPacketHandler(node);
	super.activate(node, register, principal);
    }
}

