/*
 * LineListTokenizerTest.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the ANTS distribution.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that (i) the above copyright notice(s)
 * appear in all copies and (ii) that both the copyright notice(s)
 * and this permission notice appear in supporting documentation,
 * and (iii) that the name(s) of the copyright holders not be used
 * in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  The
 * copyright holders make no representations about the suitability
 * of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 * 
 * The maintainers of this code ask, but do not require, that
 * improvements and bug fixes be emailed to ants-dist@cs.utah.edu
 *         
 */

package tests;

import ants.util.LineListTokenizer;
import ants.util.LineListException;

import java.io.PrintStream; // XXX
import java.io.FileOutputStream; // XXX
import java.io.FileNotFoundException; // XXX
import java.io.IOException; // XXX

public class LineListTokenizerTest
    extends Tester
{
    private static void createInterstingFile(String name)
	throws IOException, FileNotFoundException
    {
	PrintStream p = new PrintStream(new FileOutputStream(name));
	
	p.println("#");
	p.println("one");
	p.println(" two two ");
	p.println("1 2  3");
	p.println("# 1 2  3");
	p.println("# - three nodes (source, router, destination)   ");
	p.println("four four 4 4");
	p.println("application 18.31.12.1 edu.utah.janos.ants.apps.ping.PingApplication -target 18.31.12.3 -iter 5 -interv 4000 -lossage 0 -prime 8000");
	p.println("node 18.31.12.3 -routes ping.routes -log 255 -consoleport 2052 -updateRoutes");
	p.println("channel 18.31.12.3 localhost:8006 -log 255");
	p.println("connect 18.31.12.1 18.31.12.2");
	p.println("connect 18.31.12.2 18.31.12.3");
	p.println("");
	p.println("5 5 5 5 5");
	p.println(" ");
	p.close();
    }
    
    public void testUnit()
	throws TestFailedException
    {
	LineListTokenizer lltSelf = null;
	
	try
	{
	    this.enterSection("Testcase file");
	    createInterstingFile("LineListTokenizerTestcase.txt");
	    
	    this.enterSection("Constructor");
	    lltSelf = new LineListTokenizer("LineListTokenizerTestcase.txt");
	}
	catch (Exception e)
	{
	    throw new TestFailedException("Creating LineListTokenizer");
	}
	
	this.enterSection("Reading testcase");
	String[] line = null;
	
	while (line != null) 
	{
	    try
	    {
		line = lltSelf.list();
		if (line != null)
		    System.out.println(line.length);
	    }
	    catch (LineListException lle)
	    {
		System.out.println("Caught LineListException: " + lle);
	    }
	}
	
	this.enterSection("Removing testcase");
	try
	{
	    Runtime.getRuntime().exec("/bin/rm LineListTokenizerTestcase.txt"); // XXX
	}
	catch (java.io.IOException ioe)
	{
	    System.out.println("Failed to remove LineListTokenizerTestcase.txt:" + ioe);
	}
    }
}
